library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity vivado_npi_wr64_v1_0_M00_AXI is
	generic (
		-- Users to add parameters here

		-- User parameters ends
		-- Do not modify the parameters beyond this line

		-- Base address of targeted slave
		C_M_TARGET_SLAVE_BASE_ADDR	: std_logic_vector	:= x"40000000";
		-- Burst Length. Supports 1, 2, 4, 8, 16, 32, 64, 128, 256 burst lengths
		C_M_AXI_BURST_LEN	: integer	:= 16;
		-- Thread ID Width
		C_M_AXI_ID_WIDTH	: integer	:= 1;
		-- Width of Address Bus
		C_M_AXI_ADDR_WIDTH	: integer	:= 32;
		-- Width of Data Bus
		C_M_AXI_DATA_WIDTH	: integer	:= 32;
		-- Width of User Write Address Bus
		C_M_AXI_AWUSER_WIDTH	: integer	:= 1;
		-- Width of User Read Address Bus
		C_M_AXI_ARUSER_WIDTH	: integer	:= 1;
		-- Width of User Write Data Bus
		C_M_AXI_WUSER_WIDTH	: integer	:= 1;
		-- Width of User Read Data Bus
		C_M_AXI_RUSER_WIDTH	: integer	:= 1;
		-- Width of User Response Bus
		C_M_AXI_BUSER_WIDTH	: integer	:= 1
	);
	port (
		-- Users to add ports here
             cmd             : in std_logic_vector(7 downto 0);
             wr_fifo_clk     : in std_logic;
             wr_fifo_wr_en   : in std_logic;
             c_cnt_burst     : in std_logic_vector(15 downto 0);
             c_cnt_repeat    : in std_logic_vector(15 downto 0);
             c_base_address  : in std_logic_vector(6 downto 0);
             wr_fifo_data    : in std_logic_vector(31 downto 0); 
             wr_npi_ready    : out std_logic;
             wr_fifo_empty   : out std_logic;
             wr_fifo_full    : out std_logic;
             hw_ver          : out std_logic_vector(31 downto 0);
             wr_npi_status0  : out std_logic_vector(31 downto 0);
             wr_npi_status1  : out std_logic_vector(31 downto 0);
             wr_npi_status2  : out std_logic_vector(31 downto 0);
             wr_npi_status3  : out std_logic_vector(31 downto 0);
             wr_npi_status4  : out std_logic_vector(31 downto 0);
             wr_npi_status5  : out std_logic_vector(31 downto 0);


		-- User ports ends
		-- Do not modify the ports beyond this line

		-- Global Clock Signal.
		M_AXI_ACLK	: in std_logic;
		-- Global Reset Singal. This Signal is Active Low
		M_AXI_ARESETN	: in std_logic;
		-- Master Interface Write Address ID
		M_AXI_AWID	: out std_logic_vector(C_M_AXI_ID_WIDTH-1 downto 0);
		-- Master Interface Write Address
		M_AXI_AWADDR	: out std_logic_vector(C_M_AXI_ADDR_WIDTH-1 downto 0);
		-- Burst length. The burst length gives the exact number of transfers in a burst
		M_AXI_AWLEN	: out std_logic_vector(7 downto 0);
		-- Burst size. This signal indicates the size of each transfer in the burst
		M_AXI_AWSIZE	: out std_logic_vector(2 downto 0);
		-- Burst type. The burst type and the size information, 
    -- determine how the address for each transfer within the burst is calculated.
		M_AXI_AWBURST	: out std_logic_vector(1 downto 0);
		-- Lock type. Provides additional information about the
    -- atomic characteristics of the transfer.
		M_AXI_AWLOCK	: out std_logic;
		-- Memory type. This signal indicates how transactions
    -- are required to progress through a system.
		M_AXI_AWCACHE	: out std_logic_vector(3 downto 0);
		-- Protection type. This signal indicates the privilege
    -- and security level of the transaction, and whether
    -- the transaction is a data access or an instruction access.
		M_AXI_AWPROT	: out std_logic_vector(2 downto 0);
		-- Quality of Service, QoS identifier sent for each write transaction.
		M_AXI_AWQOS	: out std_logic_vector(3 downto 0);
		-- Optional User-defined signal in the write address channel.
		M_AXI_AWUSER	: out std_logic_vector(C_M_AXI_AWUSER_WIDTH-1 downto 0);
		-- Write address valid. This signal indicates that
    -- the channel is signaling valid write address and control information.
		M_AXI_AWVALID	: out std_logic;
		-- Write address ready. This signal indicates that
    -- the slave is ready to accept an address and associated control signals
		M_AXI_AWREADY	: in std_logic;
		-- Master Interface Write Data.
		M_AXI_WDATA	: out std_logic_vector(C_M_AXI_DATA_WIDTH-1 downto 0);
		-- Write strobes. This signal indicates which byte
    -- lanes hold valid data. There is one write strobe
    -- bit for each eight bits of the write data bus.
		M_AXI_WSTRB	: out std_logic_vector(C_M_AXI_DATA_WIDTH/8-1 downto 0);
		-- Write last. This signal indicates the last transfer in a write burst.
		M_AXI_WLAST	: out std_logic;
		-- Optional User-defined signal in the write data channel.
		M_AXI_WUSER	: out std_logic_vector(C_M_AXI_WUSER_WIDTH-1 downto 0);
		-- Write valid. This signal indicates that valid write
    -- data and strobes are available
		M_AXI_WVALID	: out std_logic;
		-- Write ready. This signal indicates that the slave
    -- can accept the write data.
		M_AXI_WREADY	: in std_logic;
		-- Master Interface Write Response.
		M_AXI_BID	: in std_logic_vector(C_M_AXI_ID_WIDTH-1 downto 0);
		-- Write response. This signal indicates the status of the write transaction.
		M_AXI_BRESP	: in std_logic_vector(1 downto 0);
		-- Optional User-defined signal in the write response channel
		M_AXI_BUSER	: in std_logic_vector(C_M_AXI_BUSER_WIDTH-1 downto 0);
		-- Write response valid. This signal indicates that the
    -- channel is signaling a valid write response.
		M_AXI_BVALID	: in std_logic;
		-- Response ready. This signal indicates that the master
    -- can accept a write response.
		M_AXI_BREADY	: out std_logic;
		-- Master Interface Read Address.
		M_AXI_ARID	: out std_logic_vector(C_M_AXI_ID_WIDTH-1 downto 0);
		-- Read address. This signal indicates the initial
    -- address of a read burst transaction.
		M_AXI_ARADDR	: out std_logic_vector(C_M_AXI_ADDR_WIDTH-1 downto 0);
		-- Burst length. The burst length gives the exact number of transfers in a burst
		M_AXI_ARLEN	: out std_logic_vector(7 downto 0);
		-- Burst size. This signal indicates the size of each transfer in the burst
		M_AXI_ARSIZE	: out std_logic_vector(2 downto 0);
		-- Burst type. The burst type and the size information, 
    -- determine how the address for each transfer within the burst is calculated.
		M_AXI_ARBURST	: out std_logic_vector(1 downto 0);
		-- Lock type. Provides additional information about the
    -- atomic characteristics of the transfer.
		M_AXI_ARLOCK	: out std_logic;
		-- Memory type. This signal indicates how transactions
    -- are required to progress through a system.
		M_AXI_ARCACHE	: out std_logic_vector(3 downto 0);
		-- Protection type. This signal indicates the privilege
    -- and security level of the transaction, and whether
    -- the transaction is a data access or an instruction access.
		M_AXI_ARPROT	: out std_logic_vector(2 downto 0);
		-- Quality of Service, QoS identifier sent for each read transaction
		M_AXI_ARQOS	: out std_logic_vector(3 downto 0);
		-- Optional User-defined signal in the read address channel.
		M_AXI_ARUSER	: out std_logic_vector(C_M_AXI_ARUSER_WIDTH-1 downto 0);
		-- Write address valid. This signal indicates that
    -- the channel is signaling valid read address and control information
		M_AXI_ARVALID	: out std_logic;
		-- Read address ready. This signal indicates that
    -- the slave is ready to accept an address and associated control signals
		M_AXI_ARREADY	: in std_logic;
		-- Read ID tag. This signal is the identification tag
    -- for the read data group of signals generated by the slave.
		M_AXI_RID	: in std_logic_vector(C_M_AXI_ID_WIDTH-1 downto 0);
		-- Master Read Data
		M_AXI_RDATA	: in std_logic_vector(C_M_AXI_DATA_WIDTH-1 downto 0);
		-- Read response. This signal indicates the status of the read transfer
		M_AXI_RRESP	: in std_logic_vector(1 downto 0);
		-- Read last. This signal indicates the last transfer in a read burst
		M_AXI_RLAST	: in std_logic;
		-- Optional User-defined signal in the read address channel.
		M_AXI_RUSER	: in std_logic_vector(C_M_AXI_RUSER_WIDTH-1 downto 0);
		-- Read valid. This signal indicates that the channel
    -- is signaling the required read data.
		M_AXI_RVALID	: in std_logic;
		-- Read ready. This signal indicates that the master can
    -- accept the read data and response information.
		M_AXI_RREADY	: out std_logic
	);
end vivado_npi_wr64_v1_0_M00_AXI;

architecture implementation of vivado_npi_wr64_v1_0_M00_AXI is
    signal axi_awvalid	: std_logic;
    signal axi_wdata	: std_logic_vector(c_m_axi_data_width-1 downto 0);
    signal axi_wlast	: std_logic;
    signal axi_wvalid	: std_logic;
    signal axi_bready	: std_logic;
    signal axi_awaddr	: std_logic_vector(c_m_axi_addr_width-1 downto 0);
    signal axi_arvalid	: std_logic;
    signal axi_rready	: std_logic;

    signal bus_error    : std_logic_vector(1 downto 0); 

    signal cnt_temp : std_logic_vector(7 downto 0);
    signal cnt_burst : std_logic_vector(15 downto 0);
    signal cnt_repeat: std_logic_vector(15 downto 0);
    signal cnt_rx : std_logic_vector(3 downto 0);
    signal step : std_logic_vector(7 downto 0);
    signal st_fifo : std_logic_vector(1 downto 0);
    signal wr_fifo_ready_s_sync, wr_fifo_full_s_sync, wr_fifo_empty_s_sync : std_logic; 
    signal wr_fifo_full_s, wr_fifo_empty_s, wr_fifo_rst_s, wr_fifo_rst_s_sync   : std_logic; 
    signal wr_fifo_wr_count_s, wr_fifo_wr_count_s_sync  : std_logic_vector(2 downto 0);
    signal sm_reset_s, cmd_wr_start, cmd_wr_start_sync  : std_logic; 
    signal wr_fifo_full_cnt : std_logic_vector(7 downto 0);
    signal wr_fifo_rd_en : std_logic;

    component fifo_generator_0
        port (
                 rst : in std_logic;
                 wr_clk : in std_logic;
                 rd_clk : in std_logic;
                 din : in std_logic_vector(31 downto 0);
                 wr_en : in std_logic;
                 rd_en : in std_logic;
                 dout : out std_logic_vector(63 downto 0);
                 full : out std_logic;
                 empty : out std_logic;
                 rd_data_count : out std_logic_vector(2 downto 0);
                 wr_data_count : out std_logic_vector(2 downto 0)
             );
    end component;
begin

    process(m_axi_aclk)										  
    begin                                                              
        if (rising_edge (m_axi_aclk)) then                               
            if (m_axi_aresetn = '0' or sm_reset_s = '1') then                                 
                step <= x"10";
                bus_error <= (others => '0');
                axi_arvalid <= '0';
                axi_wvalid <= '0'; 
                cnt_burst <= (others => '0'); 
                cnt_repeat <= (others => '0'); 
            else                                                           
                case step is
                    when x"10" =>
                        if cmd_wr_start = '1' then
                            if wr_fifo_wr_count_s_sync > "000" and wr_fifo_ready_s_sync = '1' then 
                                axi_awvalid <= '1';
                                step <= x"20";
                            end if;
                        end if;
                    when x"20" =>
                        if m_axi_awready = '1' then 
                            axi_awvalid <= '0'; 
                            axi_wvalid <= '1'; 
                            cnt_temp <= (others => '0');
                            step <= x"30";
                        end if;
                    when x"30" =>
                        if m_axi_wready = '1' then
                            if cnt_temp = x"3e" then
                                axi_wlast <= '1';
                                cnt_temp <= cnt_temp + 1;
                            elsif cnt_temp = x"3f" then
                                axi_wlast <= '0';
                                axi_wvalid <= '0'; 
                                step <= x"40";
                            else
                                axi_wlast <= '0';
                                cnt_temp <= cnt_temp + 1;
                            end if;
                        end if;
                    when x"40" =>
                        if m_axi_bvalid = '1' then 
                            if m_axi_bresp = "00" then	
                                step <= x"45";
                            else
                                bus_error <= m_axi_bresp;
                                step <= x"80";
                            end if;
                        end if;
                    when x"45" =>
                        if m_axi_bvalid = '0' then 
                            step <= x"50";
                        end if;
                    when x"50" =>
                        if cnt_burst = (c_cnt_burst - 1) then
                            cnt_burst <= (others => '0');
                            if c_cnt_repeat /= x"ffff" then
                                if cnt_repeat = (c_cnt_repeat - 1) then
                                    cnt_repeat <= (others => '0');
                                    step <= x"70";
                                else
                                    cnt_repeat <= cnt_repeat + 1;
                                    step <= x"10";
                                end if;
                            else
                                step <= x"10";
                            end if;
                        else
                            cnt_burst <= cnt_burst + 1;
                            step <= x"10";
                        end if;
                    when x"70" =>
                        step <= x"70";
                    when x"80" => 
                        step <= x"80";
                    when others => 
                        step <= x"a0";
                end case;
            end if;
        end if;
    end process;

    process(m_axi_aclk)
    begin
        if m_axi_aclk'event and m_axi_aclk = '1' then
            if m_axi_aresetn = '0' or wr_fifo_rst_s = '1' then
                st_fifo <= "00";
                wr_fifo_ready_s_sync <= '0';
            else
                case st_fifo is
                    when "00" =>
                        if wr_fifo_full_s_sync = '0' and wr_fifo_empty_s_sync = '1' then
                            wr_fifo_ready_s_sync <= '1';
                            st_fifo <= "01";
                        end if;
                    when "01" =>
                        st_fifo <= "01";
                    when others => null;
                end case;
            end if;
        end if;
    end process;

    process(wr_fifo_clk)
    begin
        if wr_fifo_clk'event and wr_fifo_clk = '1' then
            cnt_rx <= cnt_rx + 1;
        end if;
    end process;


    process(m_axi_aclk)
    begin
        if m_axi_aclk'event and m_axi_aclk = '1' then
            wr_fifo_wr_count_s_sync <=  wr_fifo_wr_count_s; 
            wr_fifo_full_s_sync <= wr_fifo_full_s;
            wr_fifo_empty_s_sync <= wr_fifo_empty_s; 
        end if;
    end process;

    -- add user logic here

    wr_npi_status0 <= wr_fifo_full_cnt & '0' & bus_error & wr_fifo_wr_count_s_sync & wr_fifo_ready_s_sync & wr_fifo_rst_s & sm_reset_s & cmd_wr_start & cnt_rx & wr_fifo_full_s & wr_fifo_empty_s_sync & step;
    wr_npi_status1 <= x"0000" & cnt_repeat;  
    wr_npi_status2 <= x"0000" & c_cnt_repeat;  
    wr_npi_status3 <= axi_awaddr; 
    wr_npi_status4 <= x"0000" &  cnt_burst; 
    wr_npi_status5 <= x"0000" &  c_cnt_burst; 
    hw_ver <= x"141203aa";

    process(wr_fifo_clk)
    begin
        if wr_fifo_clk'event and wr_fifo_clk = '1' then
            wr_fifo_rst_s_sync <= wr_fifo_rst_s;
            cmd_wr_start_sync <= cmd_wr_start;
        end if;
    end process;

    wr_fifo_rd_en <= m_axi_wready and axi_wvalid;
    async_fifo_64kb_0 : fifo_generator_0
    port map (
                 rst => wr_fifo_rst_s_sync,
                 wr_clk => wr_fifo_clk,
                 din => wr_fifo_data,
                 wr_en => wr_fifo_wr_en,
                 rd_clk => m_axi_aclk,
                 rd_en => wr_fifo_rd_en, 
                 dout => axi_wdata,
                 full => wr_fifo_full_s,
                 empty => wr_fifo_empty_s,
                 rd_data_count => open,
                 wr_data_count => wr_fifo_wr_count_s 
             );

    -- user logic ends
    wr_npi_ready <= cmd_wr_start;   
    wr_fifo_empty <= wr_fifo_empty_s;   
    wr_fifo_full <= wr_fifo_full_s;   


    process(wr_fifo_clk)
    begin
        if wr_fifo_clk'event and wr_fifo_clk = '1' then
            if wr_fifo_rst_s = '1' then 
                wr_fifo_full_cnt <= (others => '0'); 
            else
                if cmd_wr_start = '1' then
                    if wr_fifo_full_s = '1' then
                        wr_fifo_full_cnt <= wr_fifo_full_cnt + 1; 
                    end if;
                end if;
            end if;
        end if;
    end process;

    -------- default value -----------
    m_axi_awid	    <= (others => '0');
    m_axi_awaddr    <= axi_awaddr; 
    m_axi_awlen	    <= x"3f"; 
    m_axi_awsize	<= "011";       -- 010 ==> 32bit, 011 ==> 64bit 
    m_axi_awburst	<= "01";
    m_axi_awlock	<= '0';
    m_axi_awcache	<= "0010";
    m_axi_awprot	<= "000";
    m_axi_awqos	    <= x"0";
    m_axi_awuser	<= (others => '1');
    m_axi_awvalid	<= axi_awvalid;   
    m_axi_wvalid	<= axi_wvalid; 
    m_axi_wdata     <= axi_wdata;	
    m_axi_wstrb	    <= x"ff"; 
    m_axi_wlast	    <= axi_wlast;
    m_axi_bready	<= axi_bready;
    m_axi_arid	    <= (others => '0');
    m_axi_araddr	<= (others => '0');
    m_axi_arlen	    <= x"00";      
    m_axi_arsize	<= "000";       
    m_axi_arburst	<= "00";
    m_axi_arlock	<= '0';
    m_axi_arcache	<= "0010";
    m_axi_arprot	<= "000";
    m_axi_arqos	    <= x"0";
    m_axi_aruser	<= (others => '1');
    m_axi_arvalid	<= '0'; 
    m_axi_rready	<= '0';

    axi_awaddr <= c_base_address & cnt_burst & "0" & "0000" & "0000";
    wr_fifo_rst_s <= '1' when cmd(7 downto 0) = x"89" else '0';
    sm_reset_s <= '1' when cmd(7 downto 0) = x"81" else '0'; 
    cmd_wr_start <= '1' when (cmd(7 downto 0) = x"93" and step /= x"70") else '0';  

    process(m_axi_aclk)                                             
    begin                                                                 
        if (rising_edge (m_axi_aclk)) then                                  
            if (m_axi_aresetn = '0' or sm_reset_s = '1') then                                 
                axi_bready <= '0';                                              
            else                                                              
                if (m_axi_bvalid = '1' and axi_bready = '0') then               
                    axi_bready <= '1';                                            
                elsif (axi_bready = '1') then                                   
                    axi_bready <= '0';                                            
                end if;                                                         
            end if;                                                           
        end if;                                                             
    end process;                                                          

end implementation;
